package com.chelvc.cloud.maintain.controller;

import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;

import com.chelvc.cloud.maintain.copier.MerchantCopier;
import com.chelvc.cloud.maintain.vo.SimpleMerchantVO;
import com.chelvc.cloud.vehicle.client.BalanceDetailClient;
import com.chelvc.cloud.vehicle.client.FavoriteClient;
import com.chelvc.cloud.vehicle.client.GoodsClient;
import com.chelvc.cloud.vehicle.client.MerchantClient;
import com.chelvc.cloud.vehicle.client.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.client.dto.BalanceDetailDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantRankDTO;
import com.chelvc.cloud.vehicle.client.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.client.param.MerchantQueryParam;
import com.chelvc.cloud.vehicle.client.param.MerchantQueryParamPaging;
import com.chelvc.cloud.vehicle.client.param.MerchantRankParam;
import com.chelvc.cloud.vehicle.client.param.OmsCartItemPagingParam;
import com.chelvc.cloud.vehicle.client.param.QueryMerchantBalanceParam;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.security.annotation.Security;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 商家接口
 *
 * @author Woody
 * @date 2023/7/20
 */
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantController {
    private final GoodsClient goodsClient;
    private final MerchantClient merchantClient;
    private final FavoriteClient favoriteClient;
    private final BalanceDetailClient balanceDetailClient;

    /**
     * 获取商家列表
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchantSort")
    public List<SimpleMerchantVO> listSimpleMerchants(@Valid MerchantQueryParam param) {
        param.setCode((param.getRegion() / 100) * 100);
        param.setBoundary((param.getRegion() / 100) * 100 + 99);
        return MerchantCopier.INSTANCE.copying(this.merchantClient.listSimpleMerchants(param));
    }

    /**
     * 分页获取商家列表
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchantPagingList")
    public Pagination<MerchantDTO> merchantPagingList(@Valid MerchantQueryParamPaging param) {
        param.setCode((param.getRegion() / 100) * 100);
        param.setBoundary((param.getRegion() / 100) * 100 + 99);
        return this.merchantClient.merchantPagingList(param);
    }

    /**
     * 根据地区获取商家排名
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchants/rank")
    public List<MerchantRankDTO> listRankMerchants(@Valid MerchantRankParam param) {
        return this.merchantClient.listUserRankMerchants(param, SessionContextHolder.getId());
    }

    /**
     * 获取商家详情
     *
     * @param id    商家ID
     * @param param 查询参数
     * @return 商家详情
     */
    @GetMapping("/merchant/{id}")
    public MerchantDetailDTO getMerchantDetail(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id,
                                               @Valid LocationQueryParam param) {
        param.setCode((param.getRegion() / 100) * 100);
        param.setBoundary((param.getRegion() / 100) * 100 + 99);
        return this.merchantClient.queryMerchantDetail(id, param);
    }

    /**
     * 收藏商家信息
     *
     * @param id 商家ID
     * @return 收藏ID
     */
    @PostMapping("/merchant/{id}/favorite")
    public Long addMerchantFavorite(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id) {
        return this.favoriteClient.addMerchantFavorite(id, "MERCHANT");
    }

    /**
     * 查询商家余额明细
     *
     * @param param    查询条件
     * @param pageNum
     * @param pageSize
     * @return 收藏ID
     */
    @GetMapping("/merchant/balance/detail")
    public Pagination<BalanceDetailDTO> queryBalanceDetail(@Validated QueryMerchantBalanceParam param,
                                                           Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        return this.balanceDetailClient.queryByMerchantId(param, pageNum, pageSize, 2);
    }


    /**
     * 我的购物车列表
     */
    @GetMapping("/mineCartList")
    public Pagination<MerchantDTO> mineCartList(@Valid OmsCartItemPagingParam param) {
        return this.merchantClient.getUserMineCartPaging(param);
    }


    /**
     * 上架商品
     *
     * @param goodsIds 商品ID集合
     */
    @PutMapping("/merchant/list-goods/{goodsIds}")
    public void listGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsClient.updateGoodsStatus(goodsIds, GoodsStatus.ONLINE);
    }

    /**
     * 下架商品
     *
     * @param goodsIds 商品ID集合
     */
    @PutMapping("/merchant/delist-goods/{goodsIds}")
    public void delistGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsClient.updateGoodsStatus(goodsIds, GoodsStatus.OFFLINE);
    }


    /**
     * 修改商家营业状态
     *
     * @param id             商家主键
     * @param businessStatus 修改参数  营业状态 0-营业中，1-休息中
     */
    @GetMapping("/merchant/updateBusinessStatus/{id}")
    public void updateMerchant(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id,
                               String businessStatus) {
        this.merchantClient.updateMerchantBusinessStatus(id, businessStatus);
    }

    /**
     * 子商户入网回调接口
     */
    @Security(enabled = false)
    @PostMapping("/merchant/apply/notify")
    public String merchantApplyNotify(HttpServletRequest request,
                                    @RequestBody SxyNotifyParam param) {
        String merchantId = request.getHeader("merchantId");
        String encryptKey = request.getHeader("encryptKey");
        param.setMerchantId(merchantId);
        param.setEncryptKey(encryptKey);
        return this.merchantClient.merchantApplyNotify(param);
    }

    /**
     * 电子签回调接口
     *
     */
    @PostMapping("/merchant/reSign/notify")
    public String merchantReSignNotify(HttpServletRequest request,
           @RequestParam("frequency") Integer frequency, @RequestBody SxyNotifyParam param) {
        String merchantId = request.getHeader("merchantId");
        String encryptKey = request.getHeader("encryptKey");
        param.setMerchantId(merchantId);
        param.setEncryptKey(encryptKey);
        return this.merchantClient.merchantReSignNotify(frequency, param);
    }
}
